package Agent::SAMFS;
#<copyright>
# ----------------------------------------------------------
# Sun Proprietary/Confidential Code
# Copyright 2001, Sun Microsystems, Inc. All rights reserved.
# ----------------------------------------------------------
#</copyright>

#  $Name:  $ 

#  $Id: SAMFS.pm,v 1.6 2004/05/25 23:14:39 bkeslin Exp $

use strict;
use Agent;
use base 'Agent';
use Util;

sub isSelectable {"Sun SAM-QFS"}

sub revision {'$Revision: 1.6 $'}
sub new {
  my($self) = Agent->new();

  bless ($self, 'Agent::SAMFS');
  return $self;
}
use vars qw($ERR );

sub RUN {
  my($agent, $static_list, $pass) = @_;
  my ($timelapse, $dc);
  my($renv) = System->get_renv();
  $DB::single = 1;
  my $processed_list = [];

  foreach my $dev ( $agent->deviceList($static_list) ) {
     next if ($dev->{active} eq "N");
     $dc++;  
     my $sys_report = $agent->INSTRUMENTATION($dev);
     my $wwn = $dev->{key};
     my $connect_errs = $sys_report->{"rc.error"};
     my $problems = PDM->get_last_event();

     my($id) = {
                deviceName=> $wwn,
                active    => $dev->{active},
                class     => "application.samfs",
                 name      => $dev->{name},
                 display   => $dev->{name},
                 category  => "samfs",
                 ip        => $dev->{ipno},
                };

     $sys_report = {} if (!$sys_report);
     $agent->copyDev($dev, $sys_report);
     my $new_rep;
     $sys_report->{'info.noActionable'} = "Frame Level Event only.";

     if ($connect_errs) {   
       $new_rep = Report->new($id, $sys_report,undef, Report::STATUS_CANNOT_CONNECT );
     } else {
       $sys_report->{'id.name'}     = $dev->{name};
       $sys_report->{"id.mgmtLevel"}= $dev->{mgmtLevel};
       $sys_report->{"id.ip"}       = $dev->{ip};
       $sys_report->{"id.ipno"}     = $dev->{ipno};
       $sys_report->{"id.wwn"}      = $dev->{key};
       $sys_report->{"id.revision"} = "4.1";
       $new_rep = Report->new($id, $sys_report);

     }
     require Health::SAMFS;
     Health::SAMFS->all_logic($new_rep); 
     PDM->saveReport($new_rep);

     my ($broke, $abort) = $agent->new_events($problems, $dev, 1);
     push(@$processed_list, $dev);
     last if ($broke && $pass == 1 || $abort);

  }

  return $processed_list;
}


sub INSTRUMENTATION {
  my($agent, $device) = @_;
  my(@s, %dev, $in, %info);
  my(%R);
  return Report->readTest($device) if (System->get_testMode());
  return \%R;
}

sub REPORT {
  my($class, $host, $r, $arg) = @_;

  my($out);
#  my($tableCnt) = $arg->{tableCnt};
#  my($rev)      = $r->{'_value'}{"info.revision"};
  my($host0)    = $host || System->hostname();
#  my($v)        = $r->{_value};
  my $c = "#CCCCFF";
  $out .= $class->reportHead("SAM-QFS", $r);
  $out .= "
   <tr><td bgcolor=$c align=right><b>Name:</td><td width=25%>&nbsp;$r->{_id}{name}</td>
       <td bgcolor=$c align=right><b>Monitored on:</td><td>&nbsp;$host0</td>

   <tr>
       <td bgcolor=$c align=right><b>ProductID:</td><td>&nbsp;Sun SAM-QFS</td>
       <td bgcolor=$c align=right><b>Vendor:</td><td>&nbsp;Sun</td>

  </table>
  
  ";
  $out .= "</table>\n";
  return $out;

}

1;
