package SO;
use strict;
#
# installs the .so in lib/auto/* by symlinking to the 
# right version of the .so.

use vars qw(%H);

# RETURN ($errors, $log, $report);

sub install {
  my($class, $perl) = @_;

  my @VERSIONS = ('sol6','sol7','sol9', 'sol9.perl58');
  my $HOME = "/opt/SUNWstade";
  my $PERL = $perl || "/opt/SUNWstade/bin/perl";

  my $BIN  = "$PERL -I/opt/SUNWstade/lib";

  my $LIST =  [
          ['Zlib'     , 'Compress/Zlib/Zlib' ],
          ['XML'      , 'XML/LibXML/LibXML' ],
          ['GD'       , 'GD/GD'   ],
          ['SSLeay'   , 'Crypt/SSLeay/SSLeay'   ],
          [ 'MD5'     , 'Crypt/MD5/MD5'      ],
          ['Blowfish' , 'Crypt/Blowfish/Blowfish' ],
          ['GDBM_File', 'GDBM_File/GDBM_File' ],
          ['Storable' , 'Storable/Storable' ],
          ['FCGI'     , 'FCGI/FCGI' ],
          ['Base64'   , 'MIME/Base64/Base64' ],
          ['IO'       , 'IO/IO' ],
        ];


  &load();
  my (%RC, $errors, $save);
  open(OLDERR, ">&STDERR");
  open(STDERR, '>/dev/null');

  foreach my $l (@$LIST ) {
     my $file = $l->[0];
     my $so = "$HOME/lib/auto/$l->[1].so";
     my ( $good);
     foreach my $ver (@VERSIONS) {
       if (-f "$so.$ver") {
          unlink $so;
          symlink "$so.$ver", $so;
          open(O, ">/tmp/try_lib");
          print O "#!$BIN\n";
          print O $H{$file};
          close(O);
          chmod 0777, "/tmp/try_lib";
          my $LOG ="/tmp/try_lib.log";
          system("/tmp/try_lib >$LOG 2>&1");
          open(O, $LOG);
          my @LINES = <O>; close(O);
          $save .= "FILE: $file: $ver \n";
          $save .= join("", @LINES);
          $RC{$file}{$ver} = $#LINES + 1;
          $good = $ver if ($#LINES < 0 && !$good);
       }
     }
     if ($good) {
        unlink $so;
        symlink "$so.$good", $so;
     } else {
        $errors .= "Error: Cannot find a good library for Perl module '$file' \n";
     }
  }
   
  open(STDERR, ">&OLDERR");
  
  my $report = "VERSION     ";
  foreach my $ver (@VERSIONS) {
    $report .= sprintf("%12.12s", $ver);
  }
  $report .= "\n";
  $report .= "============================================================\n";
  
  foreach my $file (sort keys %RC) {
    my $F = $RC{$file};
    $report .= sprintf("%-12.12s", $file);
    foreach my $ver (@VERSIONS) {
       $report .= sprintf("%12.12s", $RC{$file}{$ver});
    }
    $report .= "\n";
  }
  $report .= "\n";

  unlink "/tmp/test_GDBM";
  unlink "/tmp/try_lib";
  unlink "/tmp/try_lib.log";

  return ($errors, $save, $report);
}


#########################################

sub load {

$H{Blowfish} =<<'EOF';
  require Crypt::Blowfish;
  
  $cipher= Crypt::Blowfish->new("x" x 16);
EOF

$H{MD5} =<<'EOF';
  use Crypt::MD5;
  $md5    = Crypt::MD5->new();
EOF

$H{SSLeay} =<<'EOF';
  require Crypt::SSLeay;
  
EOF

$H{Dumper} =<<'EOF';
  require Data::Dumper;
  $a = Data::Dumper::Dumper( { x => '2'} );
EOF

$H{FCGI} =<<'EOF';
  require FCGI;
EOF


$H{GD} =<<'EOF';
  require GD;
EOF

$H{GDBM_File} =<<'EOF';
  use GDBM_File;
  my $F = "/tmp/test_GDBM";
  unlink $F;
  tie %hash, 'GDBM_File', $F,  &GDBM_WRCREAT, 0640;
  $hash{this} = "that";
  if (!-f $F) {
     print "ERROR: This library does nothing!\n";
  }
EOF

$H{IO} =<<'EOF';
  require IO;
  require IO::Handle;
  $fh = new IO::Handle;
EOF

$H{Base64} =<<'EOF';
  require MIME::Base64;
  my $encoded = MIME::Base64::encode("TEST");
EOF

$H{Storable} =<<'EOF';
  require Storable;
  $str = Storable::freeze({x => "y"});
EOF

$H{XML} =<<'EOF';
  require XML::LibXML;
  my $parser = XML::LibXML->new();
EOF

$H{Zlib} =<<'EOF';
  if ($] >= 5.006) {
    require Compress::Zlib;
    $dest = Compress::Zlib::memGzip("REST");
  }
EOF

  
}

1;
